Triggers = {}
HillSequence = {}
Hills = {}
CurrentHill = nil
JumpInterval = 20 * 30
HillLight = nil
Fail = false

-- Polygon mnemonics don't work during assignment :C
PolyTypeNormal = 0
PolyTypeHill = 3

function Triggers.init()
    -- Create new light to indicate the hill
    HillLight = Lights.new("normal")
    HillLight.states["primary active"].intensity = 1
    HillLight.states["primary active"].period = 15
    HillLight.states["primary active"].light_function = "constant"
    HillLight.states["secondary active"].intensity = 0
    HillLight.states["secondary active"].period = 15
    HillLight.states["secondary active"].light_function = "constant"

    hills = {}
    slate_bases = {}
    red_bases = {}
    purple_bases = {}
    yellow_bases = {}
    white_bases = {}
    orange_bases = {}
    blue_bases = {}
    green_bases = {}

    for p in Polygons() do
        if p.type == "hill" then
            hills[#hills+1] = p
            p.type = PolyTypeNormal
        elseif p.type == "base" and p.permutation == 0 then
            slate_bases[#slate_bases+1] = p
            p.type = PolyTypeNormal
        elseif p.type == "base" and p.permutation == 1 then
            red_bases[#red_bases+1] = p
            p.type = PolyTypeNormal
        elseif p.type == "base" and p.permutation == 2 then
            purple_bases[#purple_bases+1] = p
            p.type = PolyTypeNormal
        elseif p.type == "base" and p.permutation == 3 then
            yellow_bases[#yellow_bases+1] = p
            p.type = PolyTypeNormal
        elseif p.type == "base" and p.permutation == 4 then
            white_bases[#white_bases+1] = p
            p.type = PolyTypeNormal
        elseif p.type == "base" and p.permutation == 5 then
            orange_bases[#orange_bases+1] = p
            p.type = PolyTypeNormal
        elseif p.type == "base" and p.permutation == 6 then
            blue_bases[#blue_bases+1] = p
            p.type = PolyTypeNormal
        elseif p.type == "base" and p.permutation == 7 then
            green_bases[#green_bases+1] = p
            p.type = PolyTypeNormal
        end
    end

    if Game.type == "netscript" then
        Game.scoring_mode = "most time"

        if #hills == 0 then
            Fail = true
        end
    end

    local hill_index = 1
    Hills[hill_index] = hills
    hill_index = hill_index + 1

    if #slate_bases > 0 then
        Hills[hill_index] = slate_bases
        hill_index = hill_index + 1
    end
    if #red_bases > 0 then
        Hills[hill_index] = red_bases
        hill_index = hill_index + 1
    end
    if #purple_bases > 0 then
        Hills[hill_index] = purple_bases
        hill_index = hill_index + 1
    end
    if #yellow_bases > 0 then
        Hills[hill_index] = yellow_bases
        hill_index = hill_index + 1
    end
    if #white_bases > 0 then
        Hills[hill_index] = white_bases
        hill_index = hill_index + 1
    end
    if #orange_bases > 0 then
        Hills[hill_index] = orange_bases
        hill_index = hill_index + 1
    end
    if #blue_bases > 0 then
        Hills[hill_index] = blue_bases
        hill_index = hill_index + 1
    end
    if #green_bases > 0 then
        Hills[hill_index] = green_bases
        hill_index = hill_index + 1
    end

    for i=1,#Hills do
        HillSequence[i] = i
    end
    shuffle(HillSequence)

    set_new_hill()
end

function Triggers.idle()
    if Fail then
        if Game.ticks == 0 then
            FailCounter = 90
            Players.print("This map doesn't have a hill :C")
        end
        FailCounter = FailCounter - 1

        if FailCounter == 0 then
            Game.over = true
        end
    else
        for p in Players() do
            p.compass.beacon = true
            p.compass.lua = true

            if p.monster.polygon.type == "hill" then
                p.compass:all_on()

                if not p.dead and Game.type == "netscript" then
                    p.points = p.points + 1
                end
            else
                p.compass.x = hill().x
                p.compass.y = hill().y
            end

            display_overlay(p)
        end

        if Game.ticks > 0 then
            if Game.ticks % JumpInterval == 0 then
                set_new_hill()
            end
        end
    end
end

function Triggers.postidle()
    if not Fail then
        Effects.new(hill().x, hill().y, hill().z + .5, hill(), "major fusion contrail")
    end
end

function shuffle(t)
	local existing_end = t[#t]
    local iterations = #t
    local j
    
    for i = iterations, 2, -1 do
        j = Game.random(i)+1
        t[i], t[j] = t[j], t[i]
    end

	if t[1] == existing_end then
		t[1], t[#t] = t[#t], t[1]
	end

end

function set_new_hill()
    if CurrentHill then
        for i=1,#Hills[HillSequence[CurrentHill]] do
            local poly = Hills[HillSequence[CurrentHill]][i]
            poly.floor.light = poly._original_light
            poly.type = PolyTypeNormal
        end

        CurrentHill = CurrentHill + 1

        if CurrentHill > #Hills then
            shuffle(HillSequence)
            CurrentHill = 1
        end

        configure_hills()
    else
        CurrentHill = 1
        configure_hills()
    end

    if Game.ticks ~= 0 then
        for p in Players() do
            p:play_sound("alarm", .9)
        end
        Players.print("New hill chosen!")
    end
end

function configure_hills()
    for i=1,#Hills[HillSequence[CurrentHill]] do
        local poly = Hills[HillSequence[CurrentHill]][i]
        poly._original_light = poly.floor.light
        poly.floor.light = HillLight
        poly.type = PolyTypeHill
    end
end

function display_overlay(p)
    if hill().z > p.z then
        p.overlays[0].icon = UpIcon
        p.overlays[0].color = "red"

        local height_diff = math.ceil(hill().z - p.z)
        p.overlays[0].text = get_text_from_height_diff(height_diff)
    elseif hill().z < p.z then
        p.overlays[0].icon = DownIcon
        p.overlays[0].color = "green"

        local height_diff = math.ceil(p.z - hill().z)
        p.overlays[0].text = get_text_from_height_diff(height_diff)
    else
        p.overlays[0].icon = nil
        p.overlays[0].text = " "
    end
end

function get_text_from_height_diff(diff)
    local txt = "="
    for i=1,diff-1 do
        txt = txt .. "="
    end
    return txt
end

function hill()
    return Hills[HillSequence[CurrentHill]][1]
end

UpIcon = [[
2
&FF0000
%111111
%%%%%%%&&%%%%%%%
%%%%%%&&&&%%%%%%
%%%%&&&&&&&&%%%%
%%&&&&&&&&&&&&%%
&&&&&&&&&&&&&&&&
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
]]

DownIcon = [[
2
&00FF00
%111111
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
&&&&&&&&&&&&&&&&
%%&&&&&&&&&&&&%%
%%%%&&&&&&&&%%%%
%%%%%%&&&&%%%%%%
%%%%%%%&&%%%%%%%
]]