Triggers = {}
PossibleHills = {}
CurrentHills = {}
OriginalHills = {}
NumPossibleHills = 0
JumpInterval = 20 * 30
MinArea = 2
Hill = nil
AdjacentHill = nil
HillLight = nil
Fail = false

-- Polygon mnemonics don't work during assignment :C
PolyTypeNormal = 0
PolyTypeHill = 3

function Triggers.init()
    -- Create new light to indicate the hill
    HillLight = Lights.new("normal")
    HillLight.states["primary active"].intensity = 1
    HillLight.states["primary active"].period = 15
    HillLight.states["primary active"].light_function = "constant"
    HillLight.states["secondary active"].intensity = 0
    HillLight.states["secondary active"].period = 15
    HillLight.states["secondary active"].light_function = "constant"

    local hill_index = 0
    for p in Polygons() do
        if p.type == "hill" then
            CurrentHills[hill_index] = p
            OriginalHills[hill_index] = p
            hill_index = hill_index + 1
            p._original_light = p.floor.light
            p.floor.light = HillLight.index

            if #PossibleHills < 1 or p.area > MinArea then
                add_possible_hill(p)
            end
        end
    end

    if Game.type == "netscript" then
        Game.scoring_mode = "most time"

        if #CurrentHills == 0 then
            Fail = true
        end
    end
end

function Triggers.idle()
    if Fail then
        if Game.ticks == 0 then
            FailCounter = 90
            Players.print("This map doesn't have a hill :C")
        end
        FailCounter = FailCounter - 1

        if FailCounter == 0 then
            Game.over = true
        end
    else
        for p in Players() do
            if p.monster.polygon.area > MinArea and p.monster.polygon.type == "normal" then
                if not contains(PossibleHills, p.monster.polygon) then
                    add_possible_hill(p.monster.polygon)
                end
            end

            p.compass.beacon = true
            p.compass.lua = true

            if p.monster.polygon.type == "hill" then
                p.compass:all_on()

                if not p.dead and Game.type == "netscript" then
                    p.points = p.points + 1
                end
            else
                p.compass.x = hill().x
                p.compass.y = hill().y
            end

            display_overlay(p)
        end

        if Game.ticks > 0 then
            if Game.ticks % JumpInterval == 0 then
                for i=0,#CurrentHills do
                    CurrentHills[i].type = PolyTypeNormal
                    CurrentHills[i].floor.light = CurrentHills[i]._original_light
                end

                select_random_hill()
            end
        end
    end
end

function Triggers.postidle()
    if not Fail then
        Effects.new(hill().x, hill().y, hill().z + .5, hill(), "major fusion contrail")
    end
end

function contains(t, poly)
    for _, value in pairs(t) do
        if value == poly then
            return true
        end
    end
    return false
end

function add_possible_hill(poly)
    PossibleHills[NumPossibleHills] = poly
    NumPossibleHills = NumPossibleHills + 1
end

function select_random_hill()
    local hill_index = 0
    local rand = nil
    rand = Game.random(NumPossibleHills)
    while contains(CurrentHills, PossibleHills[rand]) do
        rand = Game.random(NumPossibleHills)
    end

    CurrentHills = {}

    CurrentHills[hill_index] = PossibleHills[rand]
    hill_index = hill_index + 1

    hill()._original_light = hill().floor.light
    hill().floor.light = HillLight.index
    hill().type = PolyTypeHill

    if contains(OriginalHills, hill()) then
        for i=0,#OriginalHills do
            local poly = OriginalHills[i]
            if hill() ~= poly then
                CurrentHills[hill_index] = poly
                hill_index = hill_index + 1

                poly._original_light = poly.floor.light
                poly.floor.light = HillLight.index
                poly.type = PolyTypeHill
            end
        end
    else 
        for aj in hill():adjacent_polygons() do
            if aj.z == hill().z and aj.type == "normal" then
                aj._original_light = aj.floor.light
                aj.floor.light = HillLight.index
                aj.type = PolyTypeHill

                CurrentHills[hill_index] = aj
                hill_index = hill_index + 1
            end
        end
    end

    if Game.ticks ~= 0 then
        for p in Players() do
            p:play_sound("alarm", .9)
        end
        Players.print("New hill chosen!")
    end
end

function display_overlay(p)
    if hill().z > p.z then
        p.overlays[0].icon = UpIcon
        p.overlays[0].color = "red"

        local height_diff = math.ceil(hill().z - p.z)
        p.overlays[0].text = get_text_from_height_diff(height_diff)
    elseif hill().z < p.z then
        p.overlays[0].icon = DownIcon
        p.overlays[0].color = "green"

        local height_diff = math.ceil(p.z - hill().z)
        p.overlays[0].text = get_text_from_height_diff(height_diff)
    else
        p.overlays[0].icon = nil
        p.overlays[0].text = " "
    end
end

function get_text_from_height_diff(diff)
    local txt = "="
    for i=1,diff-1 do
        txt = txt .. "="
    end
    return txt
end

function hill()
    return CurrentHills[0]
end

UpIcon = [[
2
&FF0000
%111111
%%%%%%%&&%%%%%%%
%%%%%%&&&&%%%%%%
%%%%&&&&&&&&%%%%
%%&&&&&&&&&&&&%%
&&&&&&&&&&&&&&&&
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
]]

DownIcon = [[
2
&00FF00
%111111
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
%%%%%&&&&&&%%%%%
&&&&&&&&&&&&&&&&
%%&&&&&&&&&&&&%%
%%%%&&&&&&&&%%%%
%%%%%%&&&&%%%%%%
%%%%%%%&&%%%%%%%
]]